"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerPhysics = void 0;
const constant_1 = require("@skeldjs/constant");
const protocol_1 = require("@skeldjs/protocol");
const Networkable_1 = require("../../Networkable");
const net_1 = require("../../utils/net");
const events_1 = require("../../events");
/**
 * Represents a player object for handling vent entering and exiting.
 *
 * See {@link PlayerPhysicsEvents} for events to listen to.
 */
class PlayerPhysics extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data, playerControl) {
        var _a;
        super(room, spawnType, netId, ownerid, flags, data);
        (_a = this.ventId) !== null && _a !== void 0 ? _a : (this.ventId = -1);
        this.ladderClimbSeqId = 0;
        this.player = this.owner;
        if (playerControl) {
            this.components = playerControl.components;
        }
    }
    get isInVent() {
        return this.ventId > -1;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.EnterVent:
                await this._handleEnterVent(rpc);
                break;
            case constant_1.RpcMessageTag.ExitVent: {
                await this._handleExitVent(rpc);
                break;
            }
            case constant_1.RpcMessageTag.ClimbLadder:
                await this._handleClimbLadder(rpc);
                break;
        }
    }
    async _handleEnterVent(rpc) {
        this._enterVent(rpc.ventId);
        await this.emit(new events_1.PlayerEnterVentEvent(this.room, this.player, rpc, rpc.ventId));
    }
    _enterVent(ventId) {
        this.ventId = ventId;
    }
    _rpcEnterVent(ventId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.EnterVentMessage(ventId)));
    }
    /**
     * Enter a vent on the map by its ID.
     *
     * Emits a {@link PlayerEnterVentEvent | `player.entervent`} event.
     *
     * @param ventId The ID of the vent to enter.
     * @example
     *```typescript
     * client.me.physics.enterVent(PolusVent.Office);
     * ```
     */
    enterVent(ventId) {
        this._enterVent(ventId);
        this.emit(new events_1.PlayerEnterVentEvent(this.room, this.player, undefined, ventId));
        this._rpcEnterVent(ventId);
    }
    async _handleExitVent(rpc) {
        this._exitVent(rpc.ventId);
        await this.emit(new events_1.PlayerExitVentEvent(this.room, this.player, rpc, rpc.ventId));
    }
    _exitVent(ventId) {
        this.ventId = -1;
    }
    _rpcExitVent(ventId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.ExitVentMessage(ventId)));
    }
    /**
     * Exit a vent (Does not have to be the same vent or in the same network
     * as the vent you entered).
     *
     * Emits a {@link PlayerExitVentEvent | `player.exitvent`} event.
     *
     * @param ventId The ID of the vent to exit.
     * @example
     *```typescript
     * client.me.physics.enterVent(PolusVent.Office);
     * ```
     */
    exitVent(ventId) {
        this._exitVent(ventId);
        this.emit(new events_1.PlayerExitVentEvent(this.room, this.player, undefined, ventId));
        this._rpcExitVent(ventId);
    }
    async _handleClimbLadder(rpc) {
        if (net_1.NetworkUtils.seqIdGreaterThan(rpc.sequenceid, this.ladderClimbSeqId, 1)) {
            this.ladderClimbSeqId = rpc.sequenceid;
            await this.emit(new events_1.PlayerClimbLadderEvent(this.room, this.player, rpc, rpc.ladderid));
        }
    }
    _rpcClimbLadder(ladderid) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.ClimbLadderMessage(ladderid, this.ladderClimbSeqId)));
    }
    /**
     * Climb a ladder on the map by its ID.
     *
     * Emits a {@link PlayerClimbLadderEvent | `player.climbladder`} event.
     *
     * @param ladderid The ID of the ladder to climb.
     */
    async climbLadder(ladderid) {
        this.ladderClimbSeqId++;
        if (this.ladderClimbSeqId > 255)
            this.ladderClimbSeqId = 0;
        this._rpcClimbLadder(ladderid);
        await this.emit(new events_1.PlayerClimbLadderEvent(this.room, this.player, undefined, ladderid));
    }
}
exports.PlayerPhysics = PlayerPhysics;
//# sourceMappingURL=PlayerPhysics.js.map