"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecurityCameraSystem = void 0;
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
/**
 * Represents a system responsible for handling players entering and leaving security cameras.
 *
 * See {@link SecurityCameraSystemEvents} for events to listen to.
 */
class SecurityCameraSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.players || (this.players = new Set);
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const num_players = reader.upacked();
        const before = new Set([...this.players]);
        this.players.clear();
        for (let i = 0; i < num_players; i++) {
            const player = this.ship.room.getPlayerByPlayerId(reader.uint8());
            if (player && !before.has(player)) {
                this._addPlayer(player, undefined);
            }
        }
        for (const player of before) {
            if (!this.players.has(player)) {
                this._removePlayer(player, undefined);
            }
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        const players = [...this.players];
        writer.upacked(players.length);
        for (let i = 0; i < players.length; i++) {
            if (players[i].playerId)
                writer.uint8(players[i].playerId);
        }
    }
    async _addPlayer(player, rpc) {
        this.players.add(player);
        this.dirty = true;
        const ev = await this.emit(new events_1.SecurityCameraJoinEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.players.delete(player);
        }
    }
    /**
     * Add a player to the security cameras.
     * @param player The player to add.
     * @example
     *```typescript
     * security.addPlayer(client.me);
     * ```
     */
    async addPlayer(player) {
        await this._addPlayer(player, undefined);
    }
    async join() {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.addPlayer(this.room.myPlayer);
        }
        else {
            await this._sendRepair(1);
        }
    }
    async _removePlayer(player, rpc) {
        this.players.delete(player);
        this.dirty = true;
        const ev = await this.emit(new events_1.SecurityCameraLeaveEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.players.add(player);
        }
    }
    /**
     * Remove a player from the security cameras.
     * @param player The player to remove.
     * @example
     *```typescript
     * security.removePlayer(client.me);
     * ```
     */
    async removePlayer(player) {
        await this._removePlayer(player, undefined);
    }
    async leave() {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.removePlayer(this.room.myPlayer);
        }
        else {
            await this._sendRepair(0);
        }
    }
    async HandleRepair(player, amount, rpc) {
        if (!player)
            return;
        if (amount === 1) {
            await this._addPlayer(player, rpc);
        }
        else {
            await this._removePlayer(player, rpc);
        }
    }
}
exports.SecurityCameraSystem = SecurityCameraSystem;
//# sourceMappingURL=SecurityCameraSystem.js.map